import os
import logging
from typing import Union
from jinja2 import Environment, FileSystemLoader
logger = logging.getLogger(__name__)


class ZapAutomationTemplate:
    def __init__(self, target_url: str, include_paths: str = "", exclude_paths: str = ""):
        """
        Renders OWASP ZAP automation configuration file.

        :param target_url: The top level url, mandatory, everything under this will be included
        :param include_paths: TBA: An optional list of regexes to include
        :param exclude_paths: TBA: An optional list of regexes to exclude
        """
        self.target_url = target_url
        self.include_paths = include_paths
        self.exclude_paths = exclude_paths

    def rendered(
            self,
            active_scan_rule_ids: Union[list, str] = None,
            passive_scan_rule_ids: Union[list, str] = None,
            active_scan_attack_strength: str = "Medium",
            active_scan_alert_threshold: str = "Medium",
            report_dir: str = "/tmp/zap/",  # nosec
            risk_levels: list = None,
            confidence_levels: list = None,
            template_id: str = "traditional-json",
            user_agent: str = ""
    ) -> str:
        """
        Render the automation template by providing rule IDs for active scans or passive scans. Rule IDs can be found at: https://www.zaproxy.org/docs/alerts/

        :param template_id: The template ID. Available template IDs are here: https://github.com/zaproxy/zap-extensions/tree/main/addOns/reports/src/main/zapHomeFiles/reports
        :param passive_scan_rule_ids: A comma separated string of one or more PASSIVE scan rule ID numbers.
        :param active_scan_rule_ids: A comma separated string of one or more ACTIVE scan rule ID numbers.
        :param active_scan_alert_threshold: The default Attack Strength for all rules, one of Low, Medium, High, Insane (not recommended), default: Medium
        :param active_scan_attack_strength: The default Alert Threshold for all rules, one of Off, Low, Medium, High, default: Medium
        :param report_dir: The directory into which the report will be written
        :param risk_levels: The risk levels to include in this report
        :param confidence_levels: The confidences to include in this report
        :param user_agent: The user agent to use in requests. A blank string will default to the default ZAP user agent.
        :return:
        """
        # Validate risk levels
        allowed_values = ["high", "medium", "low", "info"]
        if risk_levels is None:
            risk_levels = allowed_values.copy()
        else:
            for level in risk_levels:
                if level not in allowed_values:
                    logger.error(f"ERROR: risk_levels must be one of the following values: {', '.join(allowed_values)}")

        # Validate confidence levels
        allowed_values = ["high", "medium", "low", "falsepositive"]
        if confidence_levels is None:
            confidence_levels = allowed_values.copy()
        else:
            for level in confidence_levels:
                if level not in allowed_values:
                    logger.error(f"ERROR: confidence_levels must be one of the following values: {', '.join(allowed_values)}")

        # Validate active_scan_default_strength
        allowed_values = ["Off", "Low", "Medium", "High"]
        if active_scan_attack_strength not in allowed_values:
            active_scan_attack_strength = "Medium"
            logger.error(f"ERROR: active_scan_default_strength must be one of the following values: {', '.join(allowed_values)}")

        # Validate active_scan_default_threshold
        allowed_values = ["Low", "Medium", "High", "Insane"]
        if active_scan_alert_threshold not in allowed_values:
            active_scan_alert_threshold = "Medium"
            logger.error(f"ERROR: active_scan_default_threshold must be one of the following values: {', '.join(allowed_values)}")

        # Template ID
        # https://github.com/zaproxy/zap-extensions/tree/main/addOns/reports/src/main/zapHomeFiles/reports
        allowed_values = ["modern", "traditional-html-plus", "traditional-html", "traditional-json", "traditional-md", "traditional-pdf", "traditional-xml"]
        if template_id not in allowed_values:
            template_id = "traditional-json"
            logger.error(f"ERROR: template_id must be one of the following values: {', '.join(allowed_values)}")

        # User Agent
        if not isinstance(user_agent, str):
            user_agent = "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:82.0) Gecko/20100101 Firefox/82.0"
            logger.error(f"ERROR: please supply a valid string for the user_agent parameter.")

        active_scan_rule_ids = get_rule_id_list(rule_ids=active_scan_rule_ids)
        passive_scan_rule_ids = get_rule_id_list(rule_ids=passive_scan_rule_ids)
        template_contents = dict(
            target_url=self.target_url,
            include_paths=self.include_paths,
            exclude_paths=self.exclude_paths,
            active_scan_rule_ids=active_scan_rule_ids,
            passive_scan_rule_ids=passive_scan_rule_ids,
            attack_strength=active_scan_attack_strength,
            alert_threshold=active_scan_alert_threshold,
            report_dir=report_dir,
            risk_levels=risk_levels,
            confidence_levels=confidence_levels,
            template_id=template_id,
            user_agent=user_agent
        )
        template_path = os.path.join(os.path.dirname(__file__))
        env = Environment(loader=FileSystemLoader(template_path), lstrip_blocks=True)  # nosec
        template = env.get_template("template.yaml.j2")
        result = template.render(t=template_contents)
        return result


def get_rule_id_list(rule_ids: Union[list, str]) -> Union[list, None]:
    """Get a list of rule IDs, given a string, a comma separated string, or a list."""
    if isinstance(rule_ids, str):
        if "," in rule_ids:
            rule_ids = rule_ids.split(",")
        else:
            rule_ids = [rule_ids]
        return rule_ids
    elif isinstance(rule_ids, list):
        return rule_ids
    elif isinstance(rule_ids, type(None)):
        return None
